@@ -14,7 +14,7 @@ use finalfusion::storage::{NdArray, Storage};
 use finalfusion::vocab::Vocab;
 use itertools::Itertools;
 use ndarray::Array1;
-use numpy::{IntoPyArray, PyArray1, PyReadonlyArray1};
+use numpy::{IntoPyArray, PyArray1, PyArray2, PyReadonlyArray1};
 use pyo3::class::iter::PyIterProtocol;
 use pyo3::exceptions::PyValueError;
 use pyo3::prelude::*;
@@ -22,7 +22,7 @@ use pyo3::types::{PyAny, PyIterator, PyTuple};
 use pyo3::{exceptions, PyMappingProtocol};
 use reductive::pq::Pq;
 #[cfg(feature = "opq")]
-use reductive::pq::{GaussianOPQ, OPQ};
+use reductive::pq::{GaussianOpq, Opq};
 use toml::{self, Value};
 
 use crate::storage::PyStorage;
@@ -238,6 +238,34 @@ impl PyEmbeddings {
         }
     }
 
+    /// embedding(words,/,out)
+    /// --
+    ///
+    /// Get the embedding for a batch of words. The embeddings are returned
+    /// along with an array that indicates for each word whether an embedding
+    /// could be found.
+    ///
+    /// If a matrix is provided through the `out` argument, embeddings are
+    /// written to that matrix. Rows corresponding to words for which no
+    /// embedding could be found are not overwritten.
+    fn embedding_batch<'py>(
+        &self,
+        py: Python<'py>,
+        words: Vec<&str>,
+        out: Option<&'py PyArray2<f32>>,
+    ) -> (&'py PyArray2<f32>, Vec<bool>) {
+        let embeddings = self.embeddings.read().unwrap();
+
+        let out = match out {
+            Some(out) => out,
+            None => PyArray2::zeros(py, (words.len(), embeddings.storage().shape().1), false),
+        };
+
+        let present = embeddings.embedding_batch_into(&words, unsafe { out.as_array_mut() });
+
+        (out, present)
+    }
+
     fn embedding_with_norm(&self, word: &str) -> Option<Py<PyTuple>> {
         let embeddings = self.embeddings.read().unwrap();
 
@@ -422,7 +450,7 @@ impl PyEmbeddings {
                 n_attempts,
                 normalize,
             ),
-            "opq" => embeddings.quantize::<OPQ>(
+            "opq" => embeddings.quantize::<Opq>(
                 py,
                 n_subquantizers,
                 n_subquantizer_bits,
@@ -430,7 +458,7 @@ impl PyEmbeddings {
                 n_attempts,
                 normalize,
             ),
-            "gaussian_opq" => embeddings.quantize::<GaussianOPQ>(
+            "gaussian_opq" => embeddings.quantize::<GaussianOpq>(
                 py,
                 n_subquantizers,
                 n_subquantizer_bits,
@@ -545,7 +573,7 @@ where
     Embeddings<VocabWrap, S>: ReadEmbeddings + MmapEmbeddings,
 {
     let f = File::open(path).map_err(|e| {
-        finalfusion::error::Error::io_error("Cannot open embeddings file for reading", e)
+        finalfusion::error::Error::read_error("Cannot open embeddings file for reading", e)
     })?;
     let mut reader = BufReader::new(f);
 
